@@ -12,16 +12,19 @@
 ##############################################################################
 import codecs
 import contextlib
+import io
 import os
 import sys
 import unittest
 import warnings
+from urllib.error import HTTPError
 
-import six
-
+import zExceptions
 import Zope2
+from AccessControl.Permissions import change_proxy_roles
 from AccessControl.SecurityManagement import newSecurityManager
 from AccessControl.SecurityManagement import noSecurityManager
+from OFS.Folder import Folder
 from Testing.makerequest import makerequest
 from Testing.testbrowser import Browser
 from Testing.ZopeTestCase import FunctionalTestCase
@@ -35,7 +38,7 @@ HERE = os.path.dirname(__file__)
 @contextlib.contextmanager
 def warning_interceptor():
     old_stderr = sys.stderr
-    sys.stderr = stream = six.StringIO()
+    sys.stderr = stream = io.StringIO()
     try:
         yield stream
     finally:
@@ -47,16 +50,23 @@ def warning_interceptor():
 
 def readf(name):
     path = os.path.join(HERE, 'tscripts', '%s.ps' % name)
-    with open(path, 'r') as f:
+    with open(path) as f:
         return f.read()
 
 
+class DummyFolder(Folder):
+    """ Stitch in an implementation for getPhysicalPath """
+
+    def getPhysicalPath(self):
+        return ()
+
+
 class PythonScriptTestBase(unittest.TestCase):
 
     def setUp(self):
         from AccessControl import ModuleSecurityInfo
-        from AccessControl.SecurityInfo import _moduleSecurity
         from AccessControl.SecurityInfo import _appliedModuleSecurity
+        from AccessControl.SecurityInfo import _moduleSecurity
         self._ms_before = _moduleSecurity.copy()
         self._ams_before = _appliedModuleSecurity.copy()
         ModuleSecurityInfo('string').declarePublic('split')  # noqa: D001
@@ -64,8 +74,8 @@ class PythonScriptTestBase(unittest.TestCase):
         newSecurityManager(None, None)
 
     def tearDown(self):
-        from AccessControl.SecurityInfo import _moduleSecurity
         from AccessControl.SecurityInfo import _appliedModuleSecurity
+        from AccessControl.SecurityInfo import _moduleSecurity
         noSecurityManager()
         _moduleSecurity.clear()
         _moduleSecurity.update(self._ms_before)
@@ -124,7 +134,7 @@ class TestPythonScriptNoAq(PythonScriptTestBase):
         import string
         params = string.ascii_letters[:26]
         sparams = ','.join(params)
-        ps = self._newPS('##parameters=%s\nreturn %s' % (sparams, sparams))
+        ps = self._newPS(f'##parameters={sparams}\nreturn {sparams}')
         res = ps(*params)
         self.assertEqual(res, tuple(params))
 
@@ -190,8 +200,7 @@ class TestPythonScriptNoAq(PythonScriptTestBase):
         self.assertEqual(res, 'a\n')
 
     def testComplexPrint(self):
-        script = 'complex_print_py%s' % sys.version_info.major
-        res = self._filePS(script)()
+        res = self._filePS('complex_print')()
         self.assertEqual(res, 'double\ndouble\nx: 1\ny: 0 1 2\n\n')
 
     def testNSBind(self):
@@ -207,19 +216,13 @@ class TestPythonScriptNoAq(PythonScriptTestBase):
         self.assertTrue(res)
 
     def testGetSize(self):
-        script = 'complex_print_py%s' % sys.version_info.major
-        f = self._filePS(script)
+        f = self._filePS('complex_print')
         self.assertEqual(f.get_size(), len(f.read()))
 
     def testBuiltinSet(self):
         res = self._newPS('return len(set([1, 2, 3, 1]))')()
         self.assertEqual(res, 3)
 
-    @unittest.skipIf(six.PY3, 'sets module does not exist in python3')
-    def testSetModule(self):
-        res = self._newPS('from sets import Set; return len(Set([1,2,3]))')()
-        self.assertEqual(res, 3)
-
     def testDateTime(self):
         res = self._newPS(
             "return DateTime('2007/12/10').strftime('%d.%m.%Y')")()
@@ -242,7 +245,7 @@ class TestPythonScriptNoAq(PythonScriptTestBase):
         self.assertEqual(ps.title, 'This is a title')
         self.assertEqual(ps.body(), 'print(foo+bar+baz)\nreturn printed\n')
         self.assertEqual(ps.params(), 'foo, bar, baz=1')
-        new_body = u"""\
+        new_body = """\
 ## Script (Python) "complete"
 ##bind container=container
 ##bind context=context
@@ -258,12 +261,7 @@ return \xe4\xe9\xee\xf6\xfc
         ps._filepath = 'fake'
         ps.PUT(ps.REQUEST, ps.REQUEST.RESPONSE)
         self.assertEqual(ps.title, 'New Title')
-        if six.PY3:
-            self.assertEqual(ps.body(), 'return \xe4\xe9\xee\xf6\xfc\n')
-        else:
-            self.assertEqual(
-                ps.body(),
-                'return \xc3\xa4\xc3\xa9\xc3\xae\xc3\xb6\xc3\xbc\n')
+        self.assertEqual(ps.body(), 'return \xe4\xe9\xee\xf6\xfc\n')
         self.assertEqual(ps.params(), 'oops')
 
     def test_PUT_bytes(self):
@@ -287,14 +285,94 @@ return \xc3\xa4\xc3\xa9\xc3\xae\xc3\xb6\xc3\xbc
         ps._filepath = 'fake'
         ps.PUT(ps.REQUEST, ps.REQUEST.RESPONSE)
         self.assertEqual(ps.title, 'New Title')
-        if six.PY3:
-            self.assertEqual(ps.body(), 'return \xe4\xe9\xee\xf6\xfc\n')
-        else:
-            self.assertEqual(
-                ps.body(),
-                'return \xc3\xa4\xc3\xa9\xc3\xae\xc3\xb6\xc3\xbc\n')
+        self.assertEqual(ps.body(), 'return \xe4\xe9\xee\xf6\xfc\n')
         self.assertEqual(ps.params(), 'oops')
 
+    def test_write(self):
+        ps = self._newPS('')
+
+        ps.write(b'return 1')
+        self.assertEqual(ps.body(), 'return 1\n')
+
+        ps.write('return 1')
+        self.assertEqual(ps.body(), 'return 1\n')
+
+    def test_factory(self):
+        from Products.PythonScripts.PythonScript import manage_addPythonScript
+
+        # Only passing the id
+        container = DummyFolder('container')
+        manage_addPythonScript(container, 'testing')
+        self.assertEqual(container.testing.getId(), 'testing')
+        self.assertEqual(container.testing.title, '')
+        self.assertIn('# Example code:', container.testing.body())
+        self.assertEqual(container.testing.params(), '')
+
+        # Passing id and a title
+        container = DummyFolder('container')
+        manage_addPythonScript(container, 'testing', title='This is a title')
+        self.assertEqual(container.testing.getId(), 'testing')
+        self.assertEqual(container.testing.title, 'This is a title')
+        self.assertIn('# Example code:', container.testing.body())
+        self.assertEqual(container.testing.params(), '')
+
+        # Passing id, title and a request that has no file
+        container = makerequest(DummyFolder('container'))
+        container.REQUEST.form = {}
+        manage_addPythonScript(container, 'testing', title='This is a title',
+                               REQUEST=container.REQUEST)
+        self.assertEqual(container.testing.getId(), 'testing')
+        self.assertEqual(container.testing.title, 'This is a title')
+        self.assertIn('# Example code:', container.testing.body())
+        self.assertEqual(container.testing.params(), '')
+
+        # Passing id, title and a request ith a file string
+        container = makerequest(DummyFolder('container'))
+        container.REQUEST.form = {'file': 'return 1'}
+        manage_addPythonScript(container, 'testing', title='This is a title',
+                               REQUEST=container.REQUEST)
+        self.assertEqual(container.testing.getId(), 'testing')
+        self.assertEqual(container.testing.title, 'This is a title')
+        self.assertEqual(container.testing.body(), 'return 1\n')
+        self.assertEqual(container.testing.params(), '')
+
+        # Passing id, title and a request with a file object
+        container = makerequest(DummyFolder('container'))
+        container.REQUEST.form = {'file': io.BytesIO(b'return 1')}
+        manage_addPythonScript(container, 'testing', title='This is a title',
+                               REQUEST=container.REQUEST)
+        self.assertEqual(container.testing.getId(), 'testing')
+        self.assertEqual(container.testing.title, 'This is a title')
+        self.assertEqual(container.testing.body(), 'return 1\n')
+        self.assertEqual(container.testing.params(), '')
+
+        # Passing id, title and a file string
+        container = makerequest(DummyFolder('container'))
+        manage_addPythonScript(container, 'testing', title='This is a title',
+                               file=b'return 1')
+        self.assertEqual(container.testing.getId(), 'testing')
+        self.assertEqual(container.testing.title, 'This is a title')
+        self.assertEqual(container.testing.body(), 'return 1\n')
+        self.assertEqual(container.testing.params(), '')
+
+        # Passing id, title and a file object
+        container = makerequest(DummyFolder('container'))
+        manage_addPythonScript(container, 'testing', title='This is a title',
+                               file=io.BytesIO(b'return 1'))
+        self.assertEqual(container.testing.getId(), 'testing')
+        self.assertEqual(container.testing.title, 'This is a title')
+        self.assertEqual(container.testing.body(), 'return 1\n')
+        self.assertEqual(container.testing.params(), '')
+
+    def testCodeIntrospection(self):
+        script = self._newPS('##parameters=a="b"')
+
+        self.assertEqual(script.__code__.co_argcount, 1)
+        self.assertEqual(
+            script.__code__.co_varnames,
+            ('a',))
+        self.assertEqual(script.__defaults__, ('b',))
+
 
 class TestPythonScriptErrors(PythonScriptTestBase):
 
@@ -343,7 +421,7 @@ class TestPythonScriptErrors(PythonScriptTestBase):
         """
         bad_identifiers = [
             'context', 'container', 'script', 'traverse_subpath',
-            ]
+        ]
         for identifier in bad_identifiers:
             with self.assertRaises(ValueError):
                 PythonScript(identifier)
@@ -373,10 +451,7 @@ class TestPythonScriptGlobals(PythonScriptTestBase):
 
     def test__name__(self):
         f = self._filePS('class.__name__')
-        if six.PY3:
-            class_name = "'script.class.__name__.<locals>.foo'>"
-        else:
-            class_name = "'script.foo'>"
+        class_name = "'script.class.__name__.<locals>.foo'>"
 
         self.assertEqual(f(), (class_name, "'string'"))
 
@@ -398,11 +473,8 @@ class TestPythonScriptGlobals(PythonScriptTestBase):
 class PythonScriptInterfaceConformanceTests(unittest.TestCase):
 
     def test_class_conforms_to_IWriteLock(self):
+        from OFS.interfaces import IWriteLock
         from zope.interface.verify import verifyClass
-        try:
-            from OFS.interfaces import IWriteLock
-        except ImportError:
-            from webdav.interfaces import IWriteLock
         verifyClass(IWriteLock, PythonScript)
 
 
@@ -411,7 +483,7 @@ class PythonScriptBrowserTests(FunctionalTestCase):
 
     def setUp(self):
         from Products.PythonScripts.PythonScript import manage_addPythonScript
-        super(PythonScriptBrowserTests, self).setUp()
+        super().setUp()
 
         Zope2.App.zcml.load_site(force=True)
 
@@ -420,13 +492,99 @@ class PythonScriptBrowserTests(FunctionalTestCase):
         manage_addPythonScript(self.app, 'py_script')
 
         self.browser = Browser()
-        self.browser.addHeader(
-            'Authorization',
-            'basic {}'.format(codecs.encode(  # noqa: P101
-                b'manager:manager_pass', 'base64').decode()))
+        pw = codecs.encode(b'manager:manager_pass', 'base64').decode()
+        self.browser.addHeader('Authorization', f'basic {pw}')
         self.browser.open('http://localhost/py_script/manage_main')
 
     def test_ZPythonScriptHTML_upload__no_file(self):
         """It renders an error message if no file is uploaded."""
         self.browser.getControl('Upload File').click()
         self.assertIn('No file specified', self.browser.contents)
+
+    def test_ZPythonScriptHTML_upload__with_file(self):
+        file_contents = b'print("hello")'
+        self.browser.getControl('file').add_file(
+            file_contents, 'text/plain', 'script.py')
+        self.browser.getControl('Upload File').click()
+
+        assert 'Saved changes.' in self.browser.contents
+
+    def test_PythonScript_proxyroles_manager(self):
+        test_role = 'Test Role'
+        self.app._addRole(test_role)
+
+        # Test the original state
+        self.assertFalse(self.app.py_script.manage_haveProxy(test_role))
+
+        # Go to the "Proxy" ZMI tab, grab the Proxy Roles select box,
+        # select the new role and submit
+        self.browser.open('http://localhost/py_script/manage_proxyForm')
+        roles_selector = self.browser.getControl(name='roles:list')
+        testrole_option = roles_selector.getControl(test_role)
+        self.assertFalse(testrole_option.selected)
+        testrole_option.selected = True
+        self.browser.getControl('Save Changes').click()
+
+        # The Python Script should now have a proxy role set
+        self.assertTrue(self.app.py_script.manage_haveProxy(test_role))
+
+    def test_PythonScript_proxyroles_nonmanager(self):
+        # This test checks an unusual configuration where roles other than
+        # Manager are allowed to change proxy roles.
+        proxy_form_url = 'http://localhost/py_script/manage_proxyForm'
+        test_role = 'Test Role'
+        self.app._addRole(test_role)
+        test_role_2 = 'Unprivileged Role'
+        self.app._addRole(test_role_2)
+        self.app.manage_permission(change_proxy_roles, ['Manager', test_role])
+
+        # Add some test users
+        uf = self.app.acl_users
+        uf.userFolderAddUser('privileged', 'priv', [test_role], [])
+        uf.userFolderAddUser('peon', 'unpriv', [test_role_2], [])
+
+        # Test the original state
+        self.assertFalse(self.app.py_script.manage_haveProxy(test_role))
+        self.assertFalse(self.app.py_script.manage_haveProxy(test_role_2))
+
+        # Attempt as unprivileged user will fail both in the browser and
+        # from trusted code
+        self.browser.login('peon', 'unpriv')
+        with self.assertRaises(HTTPError):
+            self.browser.open(proxy_form_url)
+
+        newSecurityManager(None, uf.getUser('peon'))
+        with self.assertRaises(zExceptions.Forbidden):
+            self.app.py_script.manage_proxy(roles=(test_role,))
+        self.assertFalse(self.app.py_script.manage_haveProxy(test_role))
+
+        # Now log in as privileged user and try to set a proxy role
+        # the privileged user does not have. This must fail.
+        self.browser.login('privileged', 'priv')
+        self.browser.open(proxy_form_url)
+        roles_selector = self.browser.getControl(name='roles:list')
+        bad_option = roles_selector.getControl(test_role_2)
+        self.assertFalse(bad_option.selected)
+        bad_option.selected = True
+        with self.assertRaises(HTTPError):
+            self.browser.getControl('Save Changes').click()
+        self.assertFalse(self.app.py_script.manage_haveProxy(test_role_2))
+
+        newSecurityManager(None, uf.getUser('privileged'))
+        with self.assertRaises(zExceptions.Forbidden):
+            self.app.py_script.manage_proxy(roles=(test_role_2,))
+        self.assertFalse(self.app.py_script.manage_haveProxy(test_role_2))
+
+        # Trying again as privileged user with a proxy role the user has
+        self.browser.open(proxy_form_url)
+        roles_selector = self.browser.getControl(name='roles:list')
+        testrole_option = roles_selector.getControl(test_role)
+        self.assertFalse(testrole_option.selected)
+        testrole_option.selected = True
+        self.browser.getControl('Save Changes').click()
+
+        # The Python Script should now have a proxy role set
+        self.assertTrue(self.app.py_script.manage_haveProxy(test_role))
+
+        # Cleanup
+        noSecurityManager()
